use Test::More tests => 1;
use strict;
use DE_EPAGES::Core::API::Script qw( RunScript );

sub Main {
    my $Installer = MY::Installer->new;
    $Installer->testLog;
}

RunScript( Sub => \&Main );

#========================================================================================
# package      MY::Installer
# description  subclass of DE_EPAGES::Core::API::BaseInstaller
#========================================================================================
package MY::Installer; ## no critic qw(ProhibitMultiplePackages)
use base 'DE_EPAGES::Core::API::BaseInstaller';

use strict;
use Log::Log4perl::Level; # $INFO
use Test::More;
use DE_EPAGES::Core::API::Log qw( GetLog );

sub new {
    my $class = shift;

    return $class->SUPER::new( CartridgeDirectory => 'dummy' );
}

# test that the "logInfo" method uses the caller package as log category
sub testLog {
    my $self = shift;

    my $appender = Log::Log4perl::Appender->new( 'MY::StringAppender', name => 'stringlog' );
    $appender->layout( Log::Log4perl::Layout::PatternLayout->new('%c: %m') );
    my $log = GetLog('');
    $log->add_appender($appender);
    $log->level($INFO);

    $self->logInfo('Hallo');
    is( $appender->string, "MY.Installer.testLog: Hallo\n", 'logInfo' );

    $log->remove_appender('stringlog');
}

#========================================================================================
# package      MY::StringAppender
# description  catches any string written to the log file
#========================================================================================
package MY::StringAppender; ## no critic qw(ProhibitMultiplePackages)

use strict;

sub new {
    my($class, @options) = @_;

    return bless { @options, 'string' => '' }, $class;
}

sub log {
    my $self = shift;
    my %params = @_;

    $self->{'string'} .= $params{message} . "\n";
}

sub string {
    my $self = shift;
    return $self->{'string'};
}

sub reset {
    my $self = shift;
    $self->{'string'} = '';
}

1;
