use strict;

use Test::Simple tests => 13;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw (
    GetCurrentDBHandle
    GetTestSite
    RunOnStore
);

# EPG-24188, EPG-25936
sub Main {
    my $dbi = GetCurrentDBHandle();
    my $driverName = $dbi->driverName;

    SKIP: {
        skip( 'dropIndexesIfExist is supported for mysql', 8 ) if $driverName ne 'mysql';

        ok( CleanUp($dbi), 'CleanUp' );
        ok( Init($dbi), 'Init' );

        my $Index;
        my @aColumns = qw( displayname street zipcode city );
        foreach my $Column ( @aColumns ) {
            $Index = 'i_' . $Column;
            $dbi->createIndex( 'testaddress', $Index, undef, [ $Column ] );
            ok( $dbi->existsIndex( 'testaddress', $Index), "Create index $Index" );
        }

        my $aIndexes = [ 'i_displayname', 'i_zipcode' ];
        $dbi->dropIndexesIfExist( 'testaddress', $aIndexes );
        foreach my $Index ( @{ $aIndexes } ) {
            ok( !$dbi->existsIndex( 'testaddress', $Index ), "Index $Index already dropped" );
        }
        ok( $dbi->existsIndex( 'testaddress', 'i_street' ), "Index $Index exists" );
        ok( $dbi->existsIndex( 'testaddress', 'i_city' ), "Index $Index exists" );

        $aIndexes = [ 'i_city' ];
        $dbi->dropIndexesIfExist( 'testaddress', $aIndexes );
        ok( !$dbi->existsIndex( 'testaddress', 'i_city' ), "Index $Index already dropped" );
        ok( $dbi->existsIndex( 'testaddress', 'i_street' ), "Index $Index exists" );

        ok( CleanUp($dbi), 'CleanUp' );
    }
}

sub CleanUp {
    my ($dbi) = @_;

    $dbi->dropTableIfExists('testaddress');

    return 1;
}

sub Init {
    my ($dbi) = @_;

    CleanUp($dbi);
    $dbi->do(<<END_SQL);
CREATE TABLE testaddress (
    addressid INTEGER PRIMARY KEY,
    displayname VARCHAR(50) NULL,
    street VARCHAR(255) NULL,
    zipcode VARCHAR(25) NULL,
    city VARCHAR(60) NULL
)
END_SQL

    return 1;
}

RunScript(
    Message => 'long',
    Sub     => sub {
        RunOnStore(
            Store => GetTestSite(),
            Sub   => \&Main
        );
    }
);

1;