use strict;
use Test::More tests => 9;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::File qw( ExistsFile );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetTestSite );

sub Main {

    my $Shop = LoadRootObject()->path('Shops/TestShop')->insertChild('Shop', {
        'Alias'=>'TestShop',
    });
    my $Style = $Shop->child('Styles')->child('MyDesign');
    ok( $Style->get('IsInvalid'), 'New style is invalid, because no CSS file exists yet' );
    is( $Style->get('Counter'), 0, 'Counter is initially 0' );

    $Style->writeCSSNoInc;
    ok( !$Style->get('IsInvalid'), 'Not invalid after writeCSSNoInc' );
    is( $Style->get('Counter'), 0, 'Counter is still 0' );

    my $FileName = $Style->get('PublicPath').'/StorefrontStyle_0.css';
    ok ExistsFile($FileName), "stylesheet $FileName exist";

    $Style->writeCSS;
    is $Style->get('Counter'), 1, 'writeCSS sets the counter';

    $Style->writeCSSNoInc;
    is $Style->get('Counter'), 1, 'writeCSSNoInc does not change the counter';

    $Style->writeCSS;
    is( $Style->get('Counter'), 2, 'writeCSS increments the counter' );
    $FileName = $Style->get('PublicPath').'/StorefrontStyle_2.css';
    ok ExistsFile($FileName), "stylesheet $FileName exists";
}

sub Init {

    Cleanup();

    LoadRootObject()->child('Shops')->insertChild('Shop', {
        'Alias'=>'TestShop',
    });
}

sub Cleanup {

    my $Folder = LoadRootObject()->child('Shops');
    $Folder->child('TestShop')->delete if $Folder->existsChild('TestShop');
}

RunScript( Debug => 1, Sub => sub {
    RunOnStore( Store => GetTestSite(), Sub => sub {
        Init();
        Main();
        Cleanup();
    });
});
