/*
 * @copyright		© Copyright 2006-2011, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages.externalcontent.gadget.twitter
 *
 * @revision		$Revision: 1.3.4.2 $
 */

jQuery.plugin('de_epages.externalcontent.gadget.twitter', {
  plugin : ['de_epages.core', 'jQuery.uid', 'jQuery.tmpl', 'jQuery.ui.widget']
}, function ($) {
  // Constants.
  var gClassNames = 'SocialWeb',
      twitterButtonTemplate = '<a class="twitter-${type}-button"'
      + 'href="https://twitter.com/{{if type==="follow"}}${username}"{{else}}share" {{if username}}data-via="${username}"{{/if}}{{/if}}'
      + '{{if type==="follow"}} data-show-count="${showcount}" data-show-screen-name="${showScreenName}"{{/if}} data-url="${url}" data-count="${count}" {{if text}}data-text="${text}"{{/if}}>'
      +	'{{if type==="follow"}}Follow ${username}{{else}}Tweet{{/if}}</a>';

  $.widget('de_epages.externalcontent.gadget.Twitter', {

    options : {
       url : ep.config.canonicalUrl ? ep.config.canonicalUrl : ep.config.webUrl,
      type : "share", 								// expects one of the following values: share, follow, profile, or search; default: share
      username : "",							// expects the username from which the tweets should be shown or the username for "via", if type "share".
      showScreenName : true,							// the Screen name of the user to attribute the Tweet to
      text : $(document).attr('title'),										// Default Tweet text
      searchterm : "ePages",							// searchterm for the search widget
      title : "Milestones",							// title of the search widget
      subject : "Fresh wind in the online shop",		// subtitle of the search widget
      shellbackground : '#8ec1da',					// color of the frame around the widget entries
      shellcolor : '#ffffff',							// color of the text in the widgetframe
      tweetbackground : '#ffffff',					// color of the background of the tweet field
      tweetcolor : '#444444',							// color of the tweets
      tweetlinks : '#1985b5',							// color of the links in tweets
      width : 250,									// width of the widget
      height: 300,									// height of the widget
      interval : 30000,							// time interval of refreshing
      behavior : 'default',						// amount of tweets to show; possible values: all, default
      count : 'none',							// String specifying the count annotation rendering. Possible values: 'none', 'vertical', 'horizontal'.
      showcount : false							// the User's followers count is not displayed with the Follow Button
    },

     _create : function () {
       var self = this,
           o = self.options,
           type = o.type;

      if ((type === 'share') || (type === 'follow')) {
      	self._twitterButton();
      }
      if ((type === 'profile') || (type === 'search')) {
      	self._twitterWidget();
      }
    },

    _twitterButton : function () {
      var self = this,
          o = self.options,
          pluginName = 'de_epages.externalcontent.gadget.twitter';

      // Render and cache template.
      if (!$.template[pluginName]) {
        $.template(pluginName, twitterButtonTemplate);
      }

      // Append button placeholder.
      self.renderedTemplate = $.tmpl(pluginName, o).appendTo(self.element.addClass(gClassNames));

      // Load Twitter Button API.
      $.ajax({
        dataType : 'script',
        url : '//platform.twitter.com/widgets.js',
        cache : true,
      })
        .done(function () {
          // Render button.
          twttr.widgets.load();
        });
    },

    _twitterWidget : function () {
      var self = this,
           o = self.options,
          twitterId = $.uid(); // For Twitter Widget API.

      // Append widget placeholder.
      this.renderedTemplate = $('<div>')
        .attr('id', twitterId)
        .appendTo(self.element.addClass(gClassNames));

      // Load Twitter Widget API.
      $.ajax({
        dataType : 'script',
        url : 'https://widgets.twimg.com/j/2/widget.js',
        cache : true,
      })
        .done(function () {
          // Configure parameters to pass to the Twitter Widget constructor.
          var params = {
            version : 2,
            id : twitterId,
            type : o.type,
            interval : o.interval,
            behavior : o.behavior,
            width : o.width,
            height : o.height,
            theme : {
              shell : {
                background : o.shellbackground,
                color : o.shellcolor
              },
              tweets : {
                background : o.tweetbackground,
                color : o.tweetcolor,
                links : o.tweetlinks
              }
            }
          };
          // extends the parameters used by both of the widgets for the ones only needed by the serach widget
          (o.type === "search") && $.extend(params, {
            search : o.searchterm,
            title : o.title,
            subject : o.subject,
            features : {
              scrollbar : true,
              loop : true,
              live : true,
              behavior: o.behavior
            }
          });
          // extends the parameters used by both of the widgets for the ones only needed by the profile widget
          (o.type === "profile") && $.extend(params, {
            rpp : 4,
            features : {
              scrollbar : true,
              loop : false,
              live : false,
              behavior : o.behavior
            }
          });
          // renders and starts the widget
          self.TWTRWidget = new TWTR.Widget(params).render();
          (o.type === 'profile') && self.TWTRWidget.setUser(o.username)
          self.TWTRWidget.start();
        });
    },

    destroy : function () {
      // Remove all traces of the widget.
      var self = this;
      self.element.removeClass(gClassNames);
      self.TWTRWidget && self.TWTRWidget.destroy();
      self.renderedTemplate.remove();
      self.Inherited(arguments);
    }

  });

});