use Test::More tests => 5;
use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore GetTestSite );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath ExistsObjectByPath LoadRootObject);
use DE_EPAGES::Fianet::API::FianetRequest qw ( DoFianetTracking );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);

my $ShopPath = '/Shops/DemoShop';
my $UserPath = $ShopPath . '/Customers/1001';
my $ROOTPATH = '/Test';


sub testFianet {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            SKIP: {

                # test tracking request
                ok( Init(), 'initialization' );
                ok( TestTrackingRequest(), 'Fianet tracking response code is "OK".' );
                ok( Cleanup(), 'cleanup');

            }
        });
}

sub TestTrackingRequest {

    my $Shop = LoadObjectByPath($ROOTPATH)->child('TestShopFianet');
    isa_ok($Shop, 'DE_EPAGES::Shop::API::Object::Shop');

    my $Order = $Shop->path('Customers/TestCustomer/Orders/1234-5678');
    isa_ok($Order, 'DE_EPAGES::Order::API::Object::CustomerOrder' );

    my $hTrackingParams = { 'REMOTE_ADDR' => '127.0.0.1' };

    return DoFianetTracking($Order, $hTrackingParams)->{'response_code'} eq 'OK';
}

sub Cleanup {
    return 1 if defined $ENV{'SKIP_CLEANUP'} && $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    my $System = LoadRootObject();
    $System->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

sub Main {
    testFianet();
    done_testing();
    return;
}

RunScript( Sub => \&Main );

1;