use strict;
use Test::More;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw (ExistsObjectByPath LoadRootObject LoadObjectByPath);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::WebInterface::API::FakeRequest;

my $ROOTPATH = '/Test';
local $ENV{'EPAGES_TESTSITE'} = 'Store' unless defined $ENV{'EPAGES_TESTSITE'};

sub Main() {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    local $DE_EPAGES::Object::API::Language::LANGUAGEID = 1;
    my $FakeRequester = DE_EPAGES::WebInterface::API::FakeRequest->new;

    my $Shop = LoadObjectByPath('/Test/TestShop/');
    ok( $Shop->alias eq 'TestShop', 'Shop loaded' );
    $Shop->set({'DomainName' => 'example.com'});

    ok ( 2 == scalar @{$Shop->languages}, 'Shop language count' );

    my $RootCategory = $Shop->get('Categories');
    my $Blog = $RootCategory->child('Blog');

    # used hash and loop to make it easier to expand test case to other content objects
    my $URLsToTest = {
        $Blog->get('WebUrl') => 'http://example.com/BLOG',
    };

    foreach my $SiteURL (keys %{$URLsToTest}) {
        ok( languageLink($FakeRequester, $SiteURL) == 1, "$SiteURL: check language link" );
        my $Can = canonicalTag($FakeRequester, $SiteURL);
        ok( $Can eq $URLsToTest->{$SiteURL}, "$SiteURL: check canonical tag." );
    }

    my $Can = canonicalTag($FakeRequester, $Blog->get('WebUrl').'&Page=2');
    ok( $Can eq '', 'check that there is no canonical tag.' );

    ok( Cleanup(), 'cleanup');
    DisconnectSite();
    done_testing();
}

sub languageLink() {
    my ($FakeRequester, $SiteURL) = @_;
    my $Response = $FakeRequester->get( $SiteURL );
    die "Can't get $SiteURL: ".$Response->status_line unless $Response->is_success;
    my $SiteContent = $Response->content;
    my $ContainsLink = ($SiteContent =~ /<link rel="alternate" hreflang=".*?" href=".*?" >/);
    return $ContainsLink;
}

sub canonicalTag() {
    my ($FakeRequester, $SiteURL) = @_;
    my $Response = $FakeRequester->get( $SiteURL );
    die "Can't get $SiteURL: ".$Response->status_line unless $Response->is_success;
    my $SiteContent = $Response->content;
    my ($Href) = ($SiteContent =~ /<link rel="canonical" href="(.*?)" \/>/);
    return $Href;
}

sub Init() {
    return 1 if LoadRootObject()->existsChild('Test') && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild('Object', {'Alias'=>'Test'});
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

sub Cleanup() {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);