use Test::More tests => 30;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Installer::API::CVSTagHandler;
use DE_EPAGES::Core::API::File qw (GetFileContent DeleteFile WriteFile DeleteDirectory);
use strict;

sub Main {
  
  SKIP: {
      skip( 'Forced skip of internal VCS tests.', 30 ) if $ENV{'SKIP_VCS_TESTS'};
      
      my $TagHandler = DE_EPAGES::Installer::API::CVSTagHandler->new();

      Cleanup($TagHandler);

      my $aChangedFiles
        = $TagHandler->getChangedFiles( 'release_test_CVSHandler_old', 'patch_test_CVSHandler_new', { 'cvspath' => 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t' } );

      ok( scalar @$aChangedFiles == 0, 'no result if tag doesnt exist.' );

      #Test if files changed, set patch tag to 1.3.2.1
      $TagHandler->command(
        'rtag',
        { '-r' => '1.3.2.1', '-F' => '' },
        [ 'patch_test_CVSHandler_new', 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ]
      );

      #get results
      $aChangedFiles
        = $TagHandler->getChangedFiles( 'release_test_CVSHandler_old', 'patch_test_CVSHandler_new', { 'cvspath' => 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t' } );
      my $aChangedFilesUnchangedContent = $TagHandler->getChangedFilesWithUnchangedContent( 'release_test_CVSHandler_old', 'patch_test_CVSHandler_new',
        { 'cvspath' => 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t' } );

      #check if changed file is find
      ok( ( ( scalar @$aChangedFiles == 1 ) and ( $aChangedFiles->[0] eq 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ) ),
        'result found in changed files.' );

      #check if no file with unchanged content is find
      ok( scalar @$aChangedFilesUnchangedContent == 0, 'no result with unchanged content exist.' );

      #Test for unchanged content set patch tag to 1.3.2.2
      $TagHandler->command(
        'rtag',
        { '-r' => '1.3.2.2', '-F' => '' },
        [ 'patch_test_CVSHandler_new', 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ]
      );

      #get results
      $aChangedFiles
        = $TagHandler->getChangedFiles( 'release_test_CVSHandler_old', 'patch_test_CVSHandler_new', { 'cvspath' => 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t' } );
      $aChangedFilesUnchangedContent = $TagHandler->getChangedFilesWithUnchangedContent( 'release_test_CVSHandler_old', 'patch_test_CVSHandler_new',
        { 'cvspath' => 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t' } );

      #check if changed file is find
      ok( ( ( scalar @$aChangedFiles == 1 ) and ( $aChangedFiles->[0] eq 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ) ),
        'result found in changed files.' );

      #check if changed file with unchanged content is find
      ok((
            ( scalar @$aChangedFilesUnchangedContent == 1 ) and
            ( $aChangedFilesUnchangedContent->[0] eq 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' )
         ),
         'result found in changed files with unchanged content.'
        );

      #removing patch tag
      $TagHandler->removeTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'patch_test_CVSHandler_new' );

      #check if result is empty
      $aChangedFiles
        = $TagHandler->getChangedFiles( 'release_test_CVSHandler_old', 'patch_test_CVSHandler_new', { 'cvspath' => 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t' } );

      #check if result is empty so this is a indirect proof that the patch tag is gone
      ok( scalar @$aChangedFiles == 0, 'patch tag is removed' );

      #check for patch affected file names
      ok( $TagHandler->patchAffectedFile('RND/epages/epages5/AllOS/Patch/src/Shared/Config/Scheduler.conf-patch'),                                'patch affected file.' );
      ok( $TagHandler->patchAffectedFile('\Cartridges\DE_EPAGES\Calendar\Data\Private\BO\sprite_Calendar.css'),                                   'patch affected file.' );
      ok( $TagHandler->patchAffectedFile('RND/epages/epages5/AllOS\Cartridges\DE_EPAGES\ProductPortal\API\Cartridge.pm'),                         'patch affected file.' );
      ok( $TagHandler->patchAffectedFile('RND/epages/epages5/AllOS\Cartridges\DE_EPAGES\Ebay\Database\XML\ActionsEbayAuction.xml'),               'patch affected file.' );
      ok( $TagHandler->patchAffectedFile('RND/epages/epages5/AllOS\Cartridges\DE_EPAGES\FactFinder\Database\mysql\Tables\factfinderchannel.sql'), 'patch affected file.' );
      ok( $TagHandler->patchAffectedFile('RND/epages/epages5/AllOS\Cartridges\DE_EPAGES\IGFS\Patches\Patch_6_12_3\patch.xml'),                    'patch affected file.' );
      ok( not( $TagHandler->patchAffectedFile('RND/epages/epages5/AllOS\Cartridges\DE_EPAGES\CreditCheck\Templates\MBO\MBO-CreditCheckAgencyConfigCustomerGroups.TabPage.en.xml') ),
        'no patch affected file.' );
      ok( not( $TagHandler->patchAffectedFile('RND/epages/epages5/AllOS/Languages/ca/Cartridges/DE_EPAGES/Presentation/Database/XML/Translation.HelptopicsShop.ca.xml') ),
        'no patch affected file.' );

      ok( not( $TagHandler->inspectionAffectedFile('RND/epages/epages5/AllOS/Patch/src/Shared/Config/Scheduler.conf-patch') ), 'no inspection affected file.' );
      ok( $TagHandler->inspectionAffectedFile('RND/epages/epages5/AllOS/Languages/ca/Cartridges/DE_EPAGES/Presentation/Database/XML/Translation.HelptopicsShop.ca.xml'),
        'inspection affected file.' );

      #get hot fix tags
      #set fictional release tag to start revision
      $TagHandler->command( 'rtag', { '-r' => '1.3', '-F' => '' }, [ 'release_testtag_88_88_88', 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ] );

      my $hInfo = {
        'engineer' => 'cbordis',
        'jiratask' => 'EPG-19048'
      };

      is_deeply( $TagHandler->getInfoForTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_88_88_88' ),
        $hInfo, 'correct engineer and jira task' );

      $TagHandler->command(
        'rtag',
        { '-r' => '1.3.2.1', '-F' => '' },
        [ 'patch_test_CVSHandler_end', 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ]
      );

      $hInfo = {
        'engineer' => 'cbordis',
        'jiratask' => 'EPG-19048'
      };

      is_deeply( $TagHandler->getInfoForTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'patch_test_CVSHandler_end' ),
        $hInfo, 'correct engineer and jira task' );

      #set fictional release tag to start revision
      $TagHandler->command( 'rtag', { '-r' => '1.3', '-F' => '' }, [ 'release_testtag_99_99_99', 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ] );

      #set fictional release tag to start revision
      $TagHandler->command( 'rtag', { '-r' => '1.3', '-F' => '' },
        [ 'release_testtag_99_99_99_1', 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ] );

      #set fictional release tag to start revision
      $TagHandler->command(
        'rtag',
        { '-r' => '1.3', '-F' => '' },
        [ 'release_testtag_99_99_99_1_1', 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ]
      );

      #set fictional release tag to start revision
      $TagHandler->command( 'rtag', { '-r' => '1.3', '-F' => '' },
        [ 'release_testtag_99_99_99_2', 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ] );

      #set fictional release tag to start revision
      $TagHandler->command( 'rtag', { '-r' => '1.3', '-F' => '' },
        [ 'release_testtag_99_99_99_3', 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ] );

      #set fictional release tag to start revision
      $TagHandler->command(
        'rtag',
        { '-r' => '1.3', '-F' => '' },
        [ 'release_testtag_99_99_99_10', 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ]
      );

      ok((
        $TagHandler->getLatestCorrespondingHotFixTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_88_88_88' ) eq
        'release_testtag_88_88_88'
        ),
        'return release tag if no hot fix tag exists.'
      );
      ok((
        $TagHandler->getLatestCorrespondingHotFixTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_99_99_99' ) eq
        'release_testtag_99_99_99_10'
        ),
        'correct result for release tag'
      );

      #revision test
      ok(( $TagHandler->getRevision( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_99_99_99_10' ) eq '1.3' ),
        'correct revision' );

      ok( $TagHandler->isDirectSuccessor( '1.13.8.3.2.4', '1.13.8.3.2.5' ), 'is direct successor correct' );

      ok( $TagHandler->isDirectSuccessor( '1.13.8.3.2.4', '1.13.8.3.2.3' ) eq '', 'is direct successor correct' );

      ok( $TagHandler->getPreviousRevision('1.13.8.3.2.1') eq '1.13.8.3', 'get previous revision correct' );

      my $aInput = [ '1.2.4.1', '1.2.4.2', '1.2.4.3', '1.2.4.4', '1.2.4.8', '1.2.4.9', '1.2.4.14' ];
      my $aOutput = { '1.2' => '1.2.4.4', '1.2.4.7' => '1.2.4.9', '1.2.4.13' => '1.2.4.14' };
      is_deeply( $TagHandler->getMergeRevisions($aInput), $aOutput, 'get merge revisions correct' );

      $aInput = [ '1.2.1', '1.2.2', '1.2.3' ];
      $aOutput = { '1' => '1.2.3' };
      is_deeply( $TagHandler->getMergeRevisions($aInput), $aOutput, 'get merge revisions correct' );

      ok( $TagHandler->getBranchForHotfix("hotfix_6_14_3_4") eq 'B_patch_6_14_3', 'get branch for hotfix correct' );

      my $hDiff = {
        'changed' => {
        'old'  => { '143' => '<           <Menu Template="BatchAction-SendFeedback"     URLAction="SendFeedback"  Position="10" />' },
        'new' => { '145' => '>           <Menu Template="BatchAction-SendFeedback"     URLAction="SendFeedback"    Position="10" />' }
        },
        'added' => {
        '149' => '>         <Template Name="BatchAction-CompleteOrder" FileName="MBO/MBO-Ebay.BatchAction-CompleteOrder.html" />',
        '145' => '>           <Menu Template="BatchAction-CompleteOrder"    URLAction="CompleteOrder"   Position="30" />'
        }
      };
      is_deeply( $TagHandler->getRevisionDiffAsHash( '1.76', '1.76.4.1', "D:\\epages\\Cartridges\\DE_EPAGES/Ebay/Database/XML/PageTypesMBO.xml" ),$hDiff,'get revision diff as hash correct');

      $TagHandler->checkout( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'EPG-23291', 'C:\\Test\\' );
      my $rString1    = GetFileContent( $ENV{'EPAGES_CARTRIDGES'} . '\\DE_EPAGES\\Installer\\t\\test_CVSHandler_Change1.txt' );
      my $rString2    = GetFileContent( $ENV{'EPAGES_CARTRIDGES'} . '\\DE_EPAGES\\Installer\\t\\test_CVSHandler_Change2.txt' );
      my $rString3    = GetFileContent( $ENV{'EPAGES_CARTRIDGES'} . '\\DE_EPAGES\\Installer\\t\\test_CVSHandler_Change3.txt' );
      my $rString4    = GetFileContent( $ENV{'EPAGES_CARTRIDGES'} . '\\DE_EPAGES\\Installer\\t\\test_CVSHandler_Change4.txt' );
      my $rString5    = GetFileContent( $ENV{'EPAGES_CARTRIDGES'} . '\\DE_EPAGES\\Installer\\t\\test_CVSHandler_Change5.txt' );
      my $rString6    = GetFileContent( $ENV{'EPAGES_CARTRIDGES'} . '\\DE_EPAGES\\Installer\\t\\test_CVSHandler_Change6.txt' );
      my $rTestString = GetFileContent( $ENV{'EPAGES_CARTRIDGES'} . '\\DE_EPAGES\\Installer\\t\\test_CVSHandler_Merge.txt' );

      foreach my $String ( @{ [ $rString1, $rString2, $rString3, $rString4, $rString5, $rString6, ] } ) {
        WriteFile( 'C:\\Test\\test_CVSHandler.txt', $String );
        $TagHandler->commit( 'EPG-23291: test case', 'C:\\Test\\test_CVSHandler.txt' );
      }
      DeleteFile('C:\\Test\\test_CVSHandler.txt');

      $TagHandler->checkout( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'EPG-23291_mergetarget', 'C:\\Test\\' );

      $TagHandler->updateChangesHash( { '1.3.46.1' => '1.3.46.4', '1.3.46.6' => '1.3.46.7' }, 'C:\\Test\\test_CVSHandler.txt' );

      my $rContent = GetFileContent('C:\\Test\\test_CVSHandler.txt');

      ok( $$rContent eq $$rTestString, 'checkout, commit, update correct' );

      DeleteTempCheckoutFiles('C:\\Test\\');

      $TagHandler->mergeChangesIntoBrach(
        'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt',
        { '1.3.46.1' => '1.3.46.4', '1.3.46.6' => '1.3.46.7' },
        'EPG-23291_mergetarget', 'test case', 'C:\\Test'
      );

      DeleteTempCheckoutFiles('C:\\Test\\');

      $TagHandler->checkout( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'EPG-23291_mergetarget', 'C:\\Test\\' );
      $rContent = GetFileContent('C:\\Test\\test_CVSHandler.txt');

      ok( $$rContent eq $$rTestString, 'merge changes into branch correct' );

      Cleanup($TagHandler);
  }
}

sub Cleanup {
  my ($TagHandler) = @_;

  #remove fictional patch tag
  $TagHandler->removeTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t',                     'patch_test_CVSHandler_new' );
  $TagHandler->removeTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t',                     'release_test_CVSHandler_old' );
  $TagHandler->removeTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_88_88_88' );
  $TagHandler->removeTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_99_99_99' );
  $TagHandler->removeTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_99_99_99_1' );
  $TagHandler->removeTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_99_99_99_1_1' );
  $TagHandler->removeTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_99_99_99_2' );
  $TagHandler->removeTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_99_99_99_3' );
  $TagHandler->removeTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_99_99_99_10' );
  $TagHandler->removeTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'patch_test_CVSHandler_end' );

  #set fictional release tag to start revision
  $TagHandler->command(
    'rtag',
    { '-r' => '1.1', '-F' => '' },
    [ 'release_test_CVSHandler_old', 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ]
  );

  $TagHandler->deleteRevisions( '1.3.46.2', '1.3.46.7', 'C:\\Test\\test_CVSHandler.txt' );
  $TagHandler->deleteRevisions( '1.3.48.2', '1.3.48.2', 'C:\\Test\\test_CVSHandler.txt' );
  DeleteTempCheckoutFiles('C:\\Test\\');

}

sub DeleteTempCheckoutFiles {
  my ($CheckoutFileDirectory) = @_;

  #add slash if necessary
  $CheckoutFileDirectory .= '\\' unless ( $CheckoutFileDirectory =~ /(\\|\/)$/ );
  DeleteFile( $CheckoutFileDirectory . '.#test_CVSHandler.txt.1.3.48.1' );
  DeleteFile( $CheckoutFileDirectory . 'test_CVSHandler.txt' );
  DeleteDirectory( $CheckoutFileDirectory . 'CVS', { 'recursive' => 1 } );
}

RunScript( Sub => \&Main );

done_testing();

1;
