#!perl -w
use Test::More tests => 6;

use strict;
use DE_EPAGES::Core::API::File qw (
    CopyFile
);
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Object::API::XMLCompare qw ( CompareXML );
use DE_EPAGES::Installer::API::LangDiffBuilder qw (
    GetTargetEncoding
    PostProcessXMLFile
    AddFilePrefix
    ChangeStructLanguage
);

sub Main {
    # 1..4
    ok(GetTargetEncoding('test.ru.xml') eq 'iso-8859-5', 'GetTargetEncoding(ru)');
    ok(GetTargetEncoding('test.xx.xml', 0) eq 'utf8', 'GetTargetEncoding(xx)');
    ok(AddFilePrefix('filename', 'prefix') eq './prefixfilename', 'AddFilePrefix(filename)');
    ok(AddFilePrefix('/this/is/a/test.ru.xml', 'prefix_') eq '/this/is/a/prefix_test.ru.xml',
        'AddFilePrefix(/path/to/filename)');

    # 5
    my $PostprocessInput = 'LangDiff_Postprocess_in.en.xml';
    my $PostprocessOutput = 'LangDiff_Postprocess_out.en.xml';
    my $PostprocessExpected = 'LangDiff_Postprocess_expected.en.xml';
    unlink($PostprocessOutput);
    CopyFile($PostprocessInput, $PostprocessOutput);
    PostProcessXMLFile($PostprocessOutput);
    ok(CompareXML($PostprocessOutput, $PostprocessExpected), 'PostProcess');
    unlink($PostprocessOutput);

    # 6
    my $ChangeLangInput = 'LangDiff_Postprocess_in.en.xml';
    my $ChangeLangOutput = 'LangDiff_Postprocess_out.de.xml';
    my $ChangeLangExpected = 'LangDiff_ChangeLanguage_expected.de.xml';
    my $ImportHandler = DE_EPAGES::Object::API::XMLStructureImportHandler->new();
    my $Structure = $ImportHandler->getXMLStructure($ChangeLangInput, { 'register_notrans' => 1} );
    ChangeStructLanguage($Structure, 'de');
    my $ExportDriver = DE_EPAGES::Object::API::XMLStructureExportDriver->new();
    unlink($ChangeLangOutput);
    $ExportDriver->exportXML($ChangeLangOutput, $Structure,
        { encoding => GetTargetEncoding($ChangeLangOutput), quotecharacter => '"' });
    ok(CompareXML($ChangeLangOutput, $ChangeLangExpected), 'ChangeLanguage');
    unlink($ChangeLangOutput);
}

RunScript( Sub => \&Main );

1;
