use Test::More;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Installer::API::Object::PatchCodeGenerator;
use strict;

sub Main {
    my $CodeGen = DE_EPAGES::Installer::API::Object::PatchCodeGenerator->new();

    my $Result = $CodeGen->getCodeForDiffInSQLFiles({
        'index_ref_01.sql' => 'index_control_01_droppedindex.sql'
    },'Sybase');
    is(OnlyDbiStatements($Result),q{$dbi->dropIndexesIfExist('amazonoffer',['i_productid']);}, 'dropped index');

    $Result = $CodeGen->getCodeForDiffInSQLFiles({
        'index_ref_01.sql' => 'index_control_01_droppedindex_twice.sql'
    },'Sybase');
    is(OnlyDbiStatements($Result),q{$dbi->dropIndexesIfExist('amazonoffer',['i_productid','i_portalconfigid_status']);}, 'dropped two indeces');


    my $Control = "
    my \$dbi = \$self->getCurrentDBHandle();

    if(\$dbi->driverName() eq \'Sybase\'){

        #changes for file index_control_01_addedindex.sql
        \$self->logInfo(\'create index amazonoffer.i_amazonitemprice\');
        \$dbi->createIndex(\'amazonoffer\', \'i_amazonitemprice\', undef, [\'amazonofferid\'])
            unless (\$dbi->existsIndex(\'amazonoffer\',\'i_amazonitemprice\'));


    }";

    $Result = $CodeGen->getCodeForDiffInSQLFiles({
        'index_ref_01.sql' => 'index_control_01_addedindex.sql'
    },'Sybase');

    is($Result,$Control, 'added index (default sort)');

    $Control = "
    my \$dbi = \$self->getCurrentDBHandle();

    if(\$dbi->driverName() eq \'Sybase\'){

        #changes for file index_control_01_addedindex_asc.sql
        \$self->logInfo(\'create index amazonoffer.i_amazonitemprice\');
        \$dbi->createIndex(\'amazonoffer\', \'i_amazonitemprice\', undef, [\'amazonofferid\'], { \'amazonofferid\' => \'ASC\' })
            unless (\$dbi->existsIndex(\'amazonoffer\',\'i_amazonitemprice\'));


    }";

    $Result = $CodeGen->getCodeForDiffInSQLFiles({
        'index_ref_01.sql' => 'index_control_01_addedindex_asc.sql'
    },'Sybase');

    is($Result,$Control, 'added index (ASC sort)');

    $Control = "
    my \$dbi = \$self->getCurrentDBHandle();

    if(\$dbi->driverName() eq \'Sybase\'){

        #changes for file index_control_01_addedindex_desc.sql
        \$self->logInfo(\'create index amazonoffer.i_amazonitemprice\');
        \$dbi->createIndex(\'amazonoffer\', \'i_amazonitemprice\', undef, [\'amazonofferid\'], { \'amazonofferid\' => \'DESC\' })
            unless (\$dbi->existsIndex(\'amazonoffer\',\'i_amazonitemprice\'));


    }";

    $Result = $CodeGen->getCodeForDiffInSQLFiles({
        'index_ref_01.sql' => 'index_control_01_addedindex_desc.sql'
    },'Sybase');

    is($Result,$Control, 'added index (DESC sort)');

    $Control = "
    my \$dbi = \$self->getCurrentDBHandle();

    if(\$dbi->driverName() eq \'Sybase\'){

        #changes for file index_control_01_addedindex_multicolumn.sql
        \$self->logInfo(\'create index amazonoffer.i_amazonitemprice\');
        \$dbi->createIndex(\'amazonoffer\', \'i_amazonitemprice\', undef, [\'parentofferid\',\'amazonofferid\'], { \'amazonofferid\' => \'ASC\' })
            unless (\$dbi->existsIndex(\'amazonoffer\',\'i_amazonitemprice\'));


    }";

    $Result = $CodeGen->getCodeForDiffInSQLFiles({
        'index_ref_01.sql' => 'index_control_01_addedindex_multicolumn.sql'
    },'Sybase');

    is($Result,$Control, 'added index on two columns');

    $Control = "
    my \$dbi = \$self->getCurrentDBHandle();

    if(\$dbi->driverName() eq \'Sybase\'){

        #changes for file index_control_01_changedindex.sql
        \$self->logInfo(\'recreate index amazonoffer.i_productid\');
        \$dbi->recreateIndex(\'amazonoffer\', \'i_productid\', undef,
            [\'productid\'],
            { \'productid\' => \'DESC\' }
        );

    }";

    $Result = $CodeGen->getCodeForDiffInSQLFiles({
        'index_ref_01.sql' => 'index_control_01_changedindex.sql'
    },'Sybase');

    is($Result,$Control, 'changed index sort order');

    $Control = "
    my \$dbi = \$self->getCurrentDBHandle();

    if(\$dbi->driverName() eq \'Sybase\'){

        #changes for file index_control_01_addedindex_unique.sql
        \$self->logInfo(\'create index amazonoffer.i_amazonitemprice\');
        \$dbi->createIndex(\'amazonoffer\', \'i_amazonitemprice\', \'UNIQUE \', [\'amazonofferid\'], { \'amazonofferid\' => \'ASC\' })
            unless (\$dbi->existsIndex(\'amazonoffer\',\'i_amazonitemprice\'));


    }";

    $Result = $CodeGen->getCodeForDiffInSQLFiles({
        'index_ref_01.sql' => 'index_control_01_addedindex_unique.sql'
    },'Sybase');

    is($Result,$Control, 'added unique index');

    $Control = "
    my \$dbi = \$self->getCurrentDBHandle();

    if(\$dbi->driverName() eq \'Sybase\'){

        #changes for file index_control_01_addedindex_unique_clustered.sql
        \$self->logInfo(\'create index amazonoffer.i_amazonitemprice\');
        \$dbi->createIndex(\'amazonoffer\', \'i_amazonitemprice\', \'UNIQUE CLUSTERED \', [\'amazonofferid\'], { \'amazonofferid\' => \'ASC\' })
            unless (\$dbi->existsIndex(\'amazonoffer\',\'i_amazonitemprice\'));


    }";

    $Result = $CodeGen->getCodeForDiffInSQLFiles({
        'index_ref_01.sql' => 'index_control_01_addedindex_unique_clustered.sql'
    },'Sybase');

    is($Result, $Control, 'added unique clustered index');

    $Control = "
    my \$dbi = \$self->getCurrentDBHandle();

    if(\$dbi->driverName() eq \'Sybase\'){

        #changes for file index_control_01_addedindex_nonclustered.sql
        \$self->logInfo(\'create index amazonoffer.i_amazonitemprice\');
        \$dbi->createIndex(\'amazonoffer\', \'i_amazonitemprice\', \'NONCLUSTERED \', [\'amazonofferid\'], { \'amazonofferid\' => \'ASC\' })
            unless (\$dbi->existsIndex(\'amazonoffer\',\'i_amazonitemprice\'));


    }";

    $Result = $CodeGen->getCodeForDiffInSQLFiles({
        'index_ref_01.sql' => 'index_control_01_addedindex_nonclustered.sql'
    },'Sybase');

    is($Result, $Control, 'added nonclustered index');

    $Control = "
    my \$dbi = \$self->getCurrentDBHandle();

    if(\$dbi->driverName() eq \'Sybase\'){

        #changes for file index_control_01_changedindex_twice.sql
        \$self->logInfo(\'recreate index amazonoffer.i_productid\');
        \$dbi->recreateIndex(\'amazonoffer\', \'i_productid\', undef,
            [\'productid\'],
            { \'productid\' => \'DESC\' }
        );
        \$self->logInfo(\'recreate index amazonoffer.i_parentofferid\');
        \$dbi->recreateIndex(\'amazonoffer\', \'i_parentofferid\', undef,
            [\'parentofferid\'],
            { \'parentofferid\' => \'ASC\' }
        );

    }";

    $Result = $CodeGen->getCodeForDiffInSQLFiles({
        'index_ref_01.sql' => 'index_control_01_changedindex_twice.sql'
    },'Sybase');

    is($Result, $Control, 'changed two indeces: sort order and clustered to default');

    $Control = "
    my \$dbi = \$self->getCurrentDBHandle();

    if(\$dbi->driverName() eq \'Sybase\'){

    }";

    $Result = $CodeGen->getCodeForDiffInSQLFiles({
        'index_ref_01.sql' => 'index_ref_01.sql'
    },'Sybase');

    is($Result, $Control, 'no changes');

    $Control = "
    my \$dbi = \$self->getCurrentDBHandle();

    if(\$dbi->driverName() eq 'mysql'){

        #changes for file addcolumn_new.sql
        die \"column position changed from addcolumn_old.sql to addcolumn_new.sql\";
        unless (\$dbi->existsTableColumn('amazonoffer', 'newfirst')) {
            \$self->logInfo('add column amazonoffer.newfirst');
            \$dbi->addTableColumnFirst('amazonoffer', 'newfirst integer null first');
        }
        unless (\$dbi->existsTableColumn('amazonoffer', 'last')) {
            \$self->logInfo('add column amazonoffer.last');
            \$dbi->addTableColumnAfterColumn('amazonoffer', 'asin', 'last integer null');
        }
        unless (\$dbi->existsTableColumn('amazonoffer', 'afterportalconfigid')) {
            \$self->logInfo('add column amazonoffer.afterportalconfigid');
            \$dbi->addTableColumnAfterColumn('amazonoffer', 'portalconfigid', 'afterportalconfigid integer null');
        }

    }";

    $Result = $CodeGen->getCodeForDiffInSQLFiles({
        'addcolumn_old.sql' => 'addcolumn_new.sql'
    },'mysql');

    is($Result, $Control, 'different variation of add table column for mysql');
}

sub OnlyDbiStatements {
    my $string = shift;
    my @matches = ($string =~ m/(\$dbi->.*;)/gm);
    $string = join "\n", @matches;
    $string =~ s/\s//g;
    return $string;
}

RunScript( Sub => \&Main );

done_testing();

1;
