/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ui.input
 *
 * @revision		$Revision: 1.39.4.2 $
 */

jQuery.plugin( 'ep.ui.input',{
	plugin: [
		'ep.core',
		'ep.dict',
		'jQuery.support.buttonFormAttribute',		
		'jQuery.support.placeholder',
		'jQuery.support.multipleUpload',
		'jQuery.fn.form',
		'jQuery.fn.class',
		'ep.ui.tooltip'
	],
	dict: [
		'ep.dict'
	]
}, function( $, undefined ){

/*
 * @dictionary		ep.dict
 *
 * @translation		{SelectFile}
 */

	// Trigger click from custom ui element on original one
 	$(document)
 		.on("click", "input + .ep-uiInput-custom:not(label *, input:disabled + *)", function(event){
 			// Use internal trigger, event is faster without jQuery() object instantiation for just an event trigger 
 			$.event.trigger("click", event.data, this.previousSibling);
 		});

	$.widget('ep.ui.Input',{

		options: {
		//	placeholder:		'text',
		//	info:				'text',
			autofocus:			false,
			big:				false
		},

		_create: function(){
			var self = this,

				elem = self.elem = self.element
					.data( 'ep-uiInput', self )
					.sub(ep),

				// stack of additional elements
				stack = self.stack = $([]),

				// get type
				type = self.type = elem[0].tagName.toLowerCase()+':',

				// classes
				elemClasses = '';

			// detailed type
			type = type==='input:' || type==='button:' ? type+elem[0].type : type;

			// get instance of dict
			self.dict = ep.dict('ep.dict');

			elem.on( 'changeAttr.uiInput', self.Proxy('_changeAttr') )
				.on( 'reset.uiInput', self.Proxy('_change') )
				.on( 'focusin.uiInput', function(){
					if( self.tooltip ){
						self.tooltip[ self.tooltip.is(':empty') ? 'hide' : 'show' ]();
					}
				})
				.on( 'focusout.uiInput',  function(){
					self._leave();
					if( self.tooltip ){
						self.tooltip.hide();
					}
				});

			if( (/file$/i).test(type) ){
				elemClasses += ' ep-uiInput-file';
				// create a button
				stack.push(
					(
						self.custom = $('<span class="ep-uiInput-custom"><span class="ep-uiInput ep-uiInput-button">' + self.dict.translate('SelectFile')+'...</span></span>')
							.insertAfter(elem)
					)[0]
				);
				// create a input to display selected file name
				stack.push(
					(
						self.fileText = $('<span class="ep-uiInput ep-uiInput-text">' + (elem.val() || '&nbsp;')  + '</span>')
							.prependTo(self.custom)
					)[0]
				);
			}

			if( self.options.big ){
				elemClasses += ' ep-uiInput-big';
			}
			if( (/(^(textarea:|select:)|(text|password|email|url|tel|date|datetime|time|number)$)/).test(type) ){
				elemClasses += ' ep-uiInput';
			}

			if( (/checkbox$/).test(type) ){
				elemClasses += ' ep-uiInput-checkbox';
				stack.push(
					(
						self.custom = $('<span class="ep-uiInput-custom ep-uiInput ep-uiInput-base ep-uiInput-field"></span>')
							.insertAfter(elem)
					)[0]
				);
			}
			else if( (/radio$/).test(type) ){
				elemClasses += ' ep-uiInput-radio';
				stack.push(
					(
						self.custom = $('<span class="ep-uiInput-custom"></span>')
							.insertAfter(elem)
					)[0]
				);
			}
			else if( (/^select:/).test(type) ){
				elemClasses += ' ep-uiInput-select';
			}
			else if( (/(^(button:|a:)|(submit|reset|button|image)$)/).test(type) ){
				elemClasses +=		(/image$/).test(type) ? ' ep-uiInput ep-uiInput-image' : ' ep-uiInput ep-uiInput-button';
				// handle reset on form
				if( (/reset$/).test(type) ){
					elem.on('click.uiInput',function(event){
							event.preventDefault();
							$(this.form).formReset();
						});
				}
				// handle submit on buttons having the form attribute
				if( (/submit$/).test(type) ){
					if( this.elem.attr('form') ){
						if( !$.support.buttonFormAttribute ){
							this.elem.on('click', function(){
								event.preventDefault ? event.preventDefault() : event.returnValue = false;
								$('form#'+$(this).attr('form')).submit();
							});
						}
					}
				}
			}
			else if( (/^textarea:$/).test(type) ){
				elemClasses += ' ep-uiInput-textarea2';
			}
			else if( (/(text|password|email|url|tel|date|datetime|time|number)$/i).test(type) ){
				elemClasses += ' ep-uiInput-text';
			}
			// change event
			if( (/(^(select:)|(file|checkbox|radio)$)/).test(type) ){
				elem.on( 'change.uiInput', self.Proxy('_change') );
			}
			// key event
			if( (/(^(textarea:)|(text|password|checkbox|radio|email|url|tel|date|datetime|time|number)$)/).test(type) ){
				elem.on( 'keyup.uiInput', self.Proxy('_change') );
			}

			// add classes for styling
			elem.addClass( elemClasses );

			// save value state
			self.lastState = [ elem[0].value, elem[0].checked, elem[0].selectedIndex ];
		},
		// init binds
		_init: function(){

			// init placeholder
			this._placeholderInit();

			// init tooltip
			this._tooltipInit();

			// handle autofocus for this elem
			if( this.elem.attr('autofocus') || this.options.autofocus ){
				if( this.placeholder ) {
					this._placeholderHide();
				}
				this.elem[0].focus();
			}
		},
		// handle addClass to the wrap
		addClass: function( name ){
			if( !(/(hover|active|focused)/).test(name) || !this.elem.is(':disabled,:readonly') ){
				this.elem.addClass(name);
			}
		},
		// handle removeClass to the wrap
		removeClass: function( name ){
			this.elem.removeClass(name);
		},

		_placeholderInit: function(){
			if( (/(text|password|email|url|tel|date|datetime|time|number)$/i).test(this.type) ){
				var valOpt = this.options.placeholder,
					valAttr = this.elem.attr('placeholder'),
					val = valOpt || valAttr;

				if( !$.support.placeholder ){
					if( !this.placeholder && val ){
						// bind
						this.elem
							.on( 'focus.uiInput', this.Proxy('_placeholderHide'))
							.on( 'blur.uiInput', this.Proxy('_placeholderShow') );
						// create a placeholder if not support by browser
						this.placeholder = $('<input>') // <input type="text"> does not work for ie default of input is text
							.attr({
								'class': this.elem.attr('class') + ' ep-uiInput-placeholder',
								'style': this.elem.attr('style')
							})
							.hide()
							.on( 'focus.uiInput', this.Proxy('_placeholderHide') )
							.insertAfter(this.elem);
						// fix prev display status
						$['_data'](this.placeholder[0],'olddisplay','inline');
						// add to stack
						this.stack.push(this.placeholder[0]);
					}
					if( this.placeholder ){
						this.placeholder
							.prop({
								value:			val || '',
								defaultValue:	val || ''
							});
						this._placeholderShow();
					}
				}
				else{
					if( val !== valAttr ){
						this.elem.attr( 'placeholder', val );
					}
				}
			}
		},

		_placeholderHide: function( event ){
			var self = this;
			if( this.placeholder ){
				var hidden = this.elem.is(':hidden');
				this.placeholder
					.hide()
					.attr('disabled',true);
				this.elem.show();

				if( event && hidden ){
					// prevent conflict of event trigger with validate #EPG-18640
					setTimeout(function(){
						self.elem[0].focus();
					},1);
				}
			}
		},

		_placeholderShow: function( event ){
			if( this.placeholder && this.placeholder.is(':hidden') ){
				if( !this.elem.val() ){
					this.elem.hide();
					this.placeholder
						.attr('disabled',false)
						.show();
				}
				else{
					this._placeholderHide( event );
				}
			}
		},

		_tooltipInit: function( buildManual ){
			if( !this.tooltip && (this.options.info || buildManual) ){
				// create a placeholder if not support by browser
				this.tooltip = ep('<div>')
					.addClass('ep-uiInput-info')
					.uiTooltip({
						interactive:	true,
						event:			'focus',
						context:		this.elem,
						orientation:	'right',
						offsetAdjust:	[0,0]
					});
				if( this.options.info ){
					this.tooltip.html(this.options.info);
				}
				// add to stack
				this.stack.push(this.tooltip[0]);
			}
		},

		_setOption: function( key, value ) {
			if( key === 'placeholder' ){
				this.elem.attr( 'placeholder', value );
			}

			return this.Inherited(arguments);
		},

		// actions if change attr by javascript
		_changeAttr: function( event, attr ){
			if( attr.hasOwnProperty('placeholder') ){
				this._placeholderInit(true);
			}
			if( attr.hasOwnProperty('value') ){
				this._change({});
			}
		},

		_stateChanged: function(){
			var oldState = this.lastState,
				newState = [ this.elem[0].value ,this.elem[0].checked, this.elem[0].selectedIndex ],
				ret = !( oldState[0] === newState[0] && oldState[1] === newState[1] && oldState[2] === newState[2] );

			this.lastState = newState;

			return ret;
		},

		_change: function( event ){
			if( event.keyCode!==9 ){
				// special for checkbox
				if( (/checkbox$/i).test(this.type) ){
					this.elem
						.trigger( 'changeAttr', {'checked':this.elem.is(':checked')} );
				}
				// special for radios
				else if( (/radio$/i).test(this.type) ){
					this.elem
						.formGroup(':radio')
							.filter(':checked')
								.trigger( 'changeAttr', {'checked':false} )
							.end()
							.filter(':not(:checked)')
								.trigger( 'changeAttr', {'checked':true} );
				}
				// display selected file
				else if( (/file$/i).test(this.type) ){
					if(this.elem.val()!=""){
						this.fileText.text( this.elem.val().replace(/^.*?fakepath[\/\\]?/,'') );
					}else{
						this.fileText.html('&nbsp;');
					}
				}
			}

			this._leave();
		},

		_leave: function(){
			this.elem.toggleClass('ui-changed', this.elem.is(':changed'));

			if( this._stateChanged() ){
				this.elem.trigger('changeValue');
				// trigger changed to form
				$(this.elem[0].form).trigger('change');
			}
		},

		destroy: function(){
			this.elem
				.removeClass(/(ep-uiInput[\w\d-]*|epWidth-\d+)/g);
			this.stack.remove();

			return this.Inherited(arguments);
		}
	});

});