/*
 * @copyright		© Copyright 2006-2011, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ui.simpledialog
 *
 * @revision		$Revision: 1.6.4.2 $
 */

jQuery.plugin( 'ep.ui.simpledialog', {
	plugin:	['jQuery.tmpl', 'jQuery.ui.widget', 'jQuery.ui.draggable', 'ep.core', 'ep.fn.contextOrientation', 'ep.ui.input'],
	tmpl:	'ep.ui.simpledialog'
}, function ($) {
	var html = $('html');

	$.widget('ep.ui.Simpledialog', {

		options : {
			autoOpen : false,
			orientation : 'bottom',
			orientationAdjust :	[0,0]
			// content
			// buttons
		},

		_create : function () {
			var self = this,
					o = self.options;

			// Add *click* listener to *open* dialog.
			self.element
				.on('click.uiSimpledialog', self.Proxy('open'))
				.sub(ep);
		},

		_init : function () {
			var self = this,
				o = self.options;

			// If *autoOpen* is true, *open* dialog.
			o.autoOpen && self.open();
		},

		_build : function () {
			// Build dialog, if it has not been built yet.
			if (!this.dialog) {
				var self = this,
						o = self.options,
						colorpicker;

				// Load template and push named tmpl elements to current instance.
				$.extend(self, $.tmpl('ep.ui.simpledialog', [o]).tmplItem('elements'));

				// Setup *dialog*.
				self.dialog
					.hide()
					.appendTo('body')
					.draggable({
						containment : 'window',
						handle : '.ep-uiSimpledialog-head'
					});

				self.dialog = ep(self.dialog);

				// Init content and buttons.
				self._setOptions({
					content : o.content,
					buttons : o.buttons
				});


				// Add *click* listener to *closeButton*. (The 'x' on the upper right.)
				self.closeButton.on('click', self.Proxy('close') );
			}
		},

		_setOption : function (key, value) {
			var self = this,
					o = self.options;

			switch (key) {

				case 'content' :
					// Remove any old *content*.
					self.content.empty();
					// Add *content*.
					self.content.append(value);
				break;

				case 'buttons' :
					// Init empty button div.
					if (self.buttons) {
						self.buttons.remove();
						self.buttons = null;
					}
					if ($.isArray(value)) {
						// Fill div with buttons.
						self.buttons = $('<div class="ep-uiSimpledialog-buttons"></div>').appendTo(self.dialog);
						$.each(value, function (i, value) {
							ep('<button type="button"></button>')
								.text(value.text)
								.on('click', value.callback)
								.uiInput()
								.appendTo(self.buttons);
						});
					}
				break;

			}

			self.Inherited(arguments);
		},


		open : function () {
			var self = this,
				o = self.options;

			self._build();
			self.dialog
				.show()
				.contextOrientation(self.element, o.orientation, o.orientationAdjust);

			// Move in other thread, required if simpledialog is created by click an element
			setTimeout(function(){
				html.on('click.uiSimpledialog', self.Proxy('_close'));
			}, 1);

			// Fire *open* event.
			self.element.trigger('open');
		},

		close : function () {
			var self = this;

			if (self.dialog) {
			// If there is no dialog, there is nothing to be closed.
				self.dialog.hide();
				html.off('click.uiSimpledialog', self.Proxy('_close'));

				// Fire *close* event.
				self.element.trigger('close');
			}
		},

		_close : function (event) {
				var self = this,
						elem	= self.element,
						target = $(event.originalTarget || event.target);

			if (!jQuery.contains( elem[0], target[0] ) && target[0]!=elem[0] && !target.is('.ep-uiSimpledialog, .ep-uiSimpledialog *')) {
			// We would not want to *close* the dialog, if the *click event* was fired from inside the dialog.
				self.close();
			}
		},

		destroy : function () {
			this.close();
			this.element.off('click.uiSimpledialog');
			this.dialog && this.dialog.remove();
			this.Inherited(arguments);
		}

	});

});