use Test::More tests => 16;

use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError );
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::MailType::API::Object::SmtpConnection;

RunScript(
    Sub => \&Main
);

sub Main {
    ConnectTestSite();

    # EPG-18009: Different SMTP server for sending newsletters

    # default connection
    my $smtp = DE_EPAGES::MailType::API::Object::SmtpConnection->new;
    ok( $smtp, 'New default connection' );
    is( $smtp->{'Server'}, 'mail', 'Default server' );
    is( $smtp->{'Port'}, '25', 'Default port' );
    is( $smtp->{'Login'}, undef, 'Default login' );
    is( $smtp->{'Password'}, undef, 'Default password' );

    # custom connection
    $smtp = DE_EPAGES::MailType::API::Object::SmtpConnection->new({
        'SMTPServer' => 'fantasymailgate.epages.de',
        'SMTPPort' => 25,
        'SMTPLogin' => 'hoppe',
        'SMTPPassword' => '123456',
    });
    ok( $smtp, 'New custom connection' );
    is( $smtp->{'Server'}, 'fantasymailgate.epages.de', 'Custom server' );
    is( $smtp->{'Port'}, '25', 'Custom port' );
    is( $smtp->{'Login'}, 'hoppe', 'Custom login' );
    is( $smtp->{'Password'}, '123456', 'Custom password' );

    # missing parameters
    eval {
        DE_EPAGES::MailType::API::Object::SmtpConnection->new({ 'SMTPPassword' => '123456' });
    };
    ok( ExistsError(), 'Error expected' );
    eval {
        DE_EPAGES::MailType::API::Object::SmtpConnection->new({ 'SMTPServer' => undef, 'SMTPPort' => 25 });
    };
    ok( ExistsError(), 'Error expected' );


    $smtp = DE_EPAGES::MailType::API::Object::SmtpConnection->new;
    ok( $smtp, 'new connection' );

    $smtp->connection( sub {
        ok( $smtp->existsConnection, 'open' );

        $smtp->send( "test mail body\nsent by $0", {
            'FROM' => 'Wolfgang Loch <wloch@epages.de>',
            'TO' => [ 'Wolfgang Loch <wloch@epages.de>' ],
            'SUBJECT' => 'test mail subject',
          });
        ok( 1, 'send' );
    });
    ok( !$smtp->existsConnection, 'close' );

    DisconnectSite();
}

1;