/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages.mediagallery.ui.memoryusage
 *
 * @revision		$Revision: 1.2.4.2 $
 */

jQuery.plugin("de_epages.mediagallery.ui.memoryusage", {
	plugin: [
		"de_epages.core",
		"jQuery.dict",
		"jQuery.tmpl",
		"jQuery.ui.widget"
	],
	dict: ["de_epages.mediagallery.dictionary"],
	tmpl: ["de_epages.mediagallery.ui.memoryusage"]
}, function($, undefined){

 /*
	*
	* @dictionary			de_epages.mediagallery.dictionary
	*
	* @translation		{MemoryUsage}
	*									{FreeMemory}
	*									{UsedMemory}
	*									{TotalMemory}
	*
	*/

	// translations
	var mediagalleryDict = $.dict("de_epages.mediagallery.dictionary"),
		translation = {
			tMemoryUsage :   mediagalleryDict.translate("MemoryUsage"),
			tFreeMemory : mediagalleryDict.translate("FreeMemory"),
			tUsedMemory : mediagalleryDict.translate("UsedMemory"),
			tTotalMemory : mediagalleryDict.translate("TotalMemory")
		},
		unit = "MB";

	$.widget( "de_epages.mediagallery.ui.Memoryusage", {

		options: {
			maxSize:      100000,
			currentUsage: 0
		},

		_create: function(){
			var o = this.options;

			$.extend(o, translation);

			this.element.addClass("ep-uiMemoryUsage");
			this.template = $.tmpl("de_epages.mediagallery.ui.memoryusage", o).appendTo(this.element);

			$.extend(this, $.tmplItem(this.template).elements);

			this.updateUsage(o.currentUsage);
		},

		updateUsage: function(currentUsage){
			var o = this.options,
				percentage = Math.ceil(100 * currentUsage / o.maxSize),
				prevCssClass = this.cssClass;

			if(percentage > 100){
				percentage = 100;
			}

			if(percentage > 90){
				this.cssClass = "PercentageCritical";
			}
			else if(percentage > 60){
				this.cssClass = "PercentageMiddle";
			}
			else{
				this.cssClass = "PercentageNormal";
			}

			this.statusBar
				.removeClass(prevCssClass)
				.addClass(this.cssClass);

			this.statusBar
				.css({width: percentage + "px"})
				.attr({title: translation.tTotalMemory +": " + o.maxSize/1000 + " " + unit + " / " + translation.tUsedMemory + ": " + Math.floor(currentUsage/1000) + " " + unit});

			this.percentage.html(percentage + "%");
			this.freeMemory.html(Math.ceil((o.maxSize - currentUsage)/1000) + " " + unit);
		},

		destroy: function(){
			this.Inherited(arguments);

			if(this.element){
				this.element.removeClass("ep-uiMemoryUsage");
				this.container.remove();
			}
		}
	});
});