use Test::More tests => 14;

use strict;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( ConnectToSite DisconnectSite );
use DE_EPAGES::Newsletter::API::Object::Newsletter;
use DE_EPAGES::NewsletterProvider::UI::System;
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );

sub Main {
    ConnectToSite();

    my $System = LoadRootObject();

    # save original newsletter SMTP configuration in order to reset the attribute values after testing is done
    my $aAttributes = [ qw( NewsletterSMTPServer NewsletterSMTPPort NewsletterSMTPLogin NewsletterSMTPPassword SMTPLogin SMTPPassword ) ];
    my $hOriginalSMTPValues = $System->get( $aAttributes );

    # send newsletter via default mail settings
    ## no critic qw(ProhibitPrivateSubroutineUse,ProtectPrivateSubs)
    my $Connection = DE_EPAGES::Newsletter::API::Object::Newsletter->_getSMTPConnection();
    ok( $Connection, 'New newsletter default connection equals the mail-server\'s configuration' );
    is( $Connection->{'Server'}, 'mail', 'Compare default newsletter and mail settings (server)' );
    is( $Connection->{'Port'}, '25', 'Compare default newsletter and mail settings (port)' );
    ok( !defined($Connection->{'Login'}), 'Compare default newsletter and mail settings (login)' );
    ok( !defined($Connection->{'Password'}), 'Compare default newsletter and mail settings (password)' );
    $Connection->connection( sub {
        ok( $Connection->existsConnection, 'Open connection' );
        $Connection->send( "Test newsletter body\nsent by $0", {
            'FROM'    => 'Dennis Hoppe <d.hoppe@epages.de>',
            'TO'      => [ 'Dennis Hoppe <d.hoppe@epages.de>' ],
            'SUBJECT' => 'Test newsletter subject',
        });
        ok( 1, 'Send newsletter' );
    });
    ok( !$Connection->existsConnection, 'Close connection' );

    # validate new newsletter settings
    $System->set({
        'NewsletterSMTPServer'   => 'mailgate.uni-weimar.de',
        'NewsletterSMTPPort'     => '587',
        'NewsletterSMTPLogin'    => 'hoppe',
        'NewsletterSMTPPassword' => '123456789',
    });
    ## no critic qw(ProhibitPrivateSubroutineUse,ProtectPrivateSubs)
    $Connection = DE_EPAGES::Newsletter::API::Object::Newsletter->_getSMTPConnection();
    is( $Connection->{'Server'}, 'mailgate.uni-weimar.de', 'Set new newsletter settings (server)' );
    is( $Connection->{'Port'}, '587', 'Set new newsletter settings (port)' );
    is( $Connection->{'Login'}, 'hoppe', 'Set new newsletter settings (login)' );
    is( $Connection->{'Password'}, '123456789', 'Set new newsletter settings (password)' );

    # mail server needs login, newsletter-server does not
    $System->set({
        'SMTPLogin' => 'hoppe',
        'SMTPPassword' => '123456',
        'NewsletterSMTPLogin' => undef,
        'NewsletterSMTPPassword' => undef,
    });
    $Connection = DE_EPAGES::Newsletter::API::Object::Newsletter->_getSMTPConnection();
    ok( !defined($Connection->{'Login'}), 'login is not defined' );
    ok( !defined($Connection->{'Password'}), 'password is not defined' );

    # reset to default values for the NewsletterSMTP* attributes as defined in System.xml
    $System->set($hOriginalSMTPValues);

    DisconnectSite();
}

RunScript(
    Sub => \&Main
);

1;