use Test::More tests => 9;

use strict;
use warnings;
use Carp qw( croak );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Core::API::File qw( CreateDirectory DeleteDirectory );

do '../Scripts/import.pl' or croak( "Cannot (do '../Scripts/import.pl'): $!" );

use Readonly;
Readonly::Scalar my $SCRIPTS_OUT       => '../Scripts/out';
Readonly::Scalar my $TESTSTORE         => 'Store';
Readonly::Array  my @INVALID_TESTSHOPS => qw( FakeShopA FakeShopB FakeShopC );
Readonly::Array  my @VALID_TESTSHOPS   => qw( TestShop1 TestShop2 TestShop3 );

sub TestImportShop {
    # set default values
    my $hOptions = {
        'storeName' => $TESTSTORE,
        'encoding'  => 'utf8',
        'filePath'  => $SCRIPTS_OUT,
        'storePath' => join('/', $SCRIPTS_OUT, $TESTSTORE),
    };
    my $import = DE_EPAGES::ShopTransfer::API::Import->new();

    {
        # redirect output to $Result for the following tests
        local *STDOUT = '';
        my $Result;

        # check for non-existing shops
        foreach my $ShopAlias ( @INVALID_TESTSHOPS ) {
            open STDOUT, '>', \$Result or croak( "Can't open STDOUT: $!" );

            ImportShop($ShopAlias, $hOptions, $import);
            chomp( $Result );
            is ($Result, "shop $ShopAlias does not exist at location $SCRIPTS_OUT/$TESTSTORE/import/$ShopAlias.", "shop $ShopAlias does not exist" );

            close STDOUT or croak( "Unable to close STDOUT: $!" );
        }

        # check existing shops
        foreach my $ShopAlias ( @VALID_TESTSHOPS ) {
            open STDOUT, '>', \$Result or croak( "Can't open STDOUT: $!" );

            ImportShop($ShopAlias, $hOptions, $import);
            chomp( $Result );
            ok ($Result =~ /import shop/sm, "shop $ShopAlias exists" );

            close STDOUT or croak( "Unable to close STDOUT: $!" );
        }
    }

    return;
}

sub Init {
    Cleanup();

    my $ExportPath = join('/', $SCRIPTS_OUT, $TESTSTORE, 'import');
    foreach my $ShopAlias ( @VALID_TESTSHOPS ) {
        CreateDirectory( join('/', $ExportPath, $ShopAlias), { recursive => 1 } );
    }

    ok( 1, 'init' );
    return;
}

sub Cleanup {
    DeleteDirectory( '../Scripts/out', { recursive => 1 } );

    ok( 1, 'cleanup' );
    return;
}

RunScript(
    Sub => sub {
        RunOnStore(
            Store => GetTestSite(),
            Sub => sub {
                Init();

                TestImportShop();

                Cleanup();
            }
        )
    },
    Debug => 1,
);

1;
