#========================================================================================
# package      DE_EPAGES::Trigger::t::BatchProcessorNested
# state        private
# description  Testing nesting of batch contexts.
#========================================================================================
package DE_EPAGES::Trigger::t::BatchProcessorNested;
use base Test::Class;

use Test::More;
use DE_EPAGES::Trigger::API::BatchProcessor qw ( RunBatchProcess );
use strict;


sub init : Test(setup) {
    $DummyBatchHandler::INSTANCES = 0;
}

sub testSimpleBatchContext : Test(no_plan) {
    RunBatchProcess('DummyBatch', sub {
        }
    );
    is( $DummyBatchHandler::INSTANCES, 1, 'dummy product batch handler instanciated once.' );
}

sub testNestedBatchContext : Test(no_plan) {
    RunBatchProcess('DummyBatch', sub {
            RunBatchProcess( 'DummyBatch', sub { } );
        }
    );
    is( $DummyBatchHandler::INSTANCES, 1, 'dummy product batch handler instanciated once, if nested.' );
}

### dummy batch handler that just counts instanciation.
## no critic (Modules::ProhibitMultiplePackages)
package DummyBatchHandler;
use strict;

use parent qw( DE_EPAGES::Trigger::API::Object::BatchProcessHandler );
use DE_EPAGES::Trigger::API::BatchProcessor qw( RegisterHandler );

our $INSTANCES = 0; ## instance count.

sub new {    ## no critic (DE_EPAGES::Style::RequireArgUnpacking)
    my $class = shift;
    $DummyBatchHandler::INSTANCES++;
    return $class->SUPER::new(@_);
}

sub OnRegisterDummyBatchHandler {
    RegisterHandler( 'DummyBatch', DummyBatchHandler->new() );
}

###### mock TriggerHook call.
no warnings qw(redefine);    ## no critic (ProhibitNoWarnings)

sub DE_EPAGES::Trigger::API::BatchProcessor::TriggerHook {
    my ($HookName) = @_;
    Test::More::is( $HookName, 'API_RegisterDummyBatchHandler', 'batch handler registration' );
    DummyBatchHandler::OnRegisterDummyBatchHandler();
}

####### Main ############
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );

RunScript(
    Debug => 1,
    Sub   => sub {
        Test::Class->runtests;
    }
);
